use egui::Hyperlink;

use super::Gui;


impl Gui {

    pub fn draw_nav(&mut self, ctx: &egui::Context, _: &mut eframe::Frame) {
        egui::TopBottomPanel::top("top_panel").show(ctx, |ui| {
            // The top panel is often a good place for a menu bar:
            egui::menu::bar(ui, |ui| {
                ui.menu_button("File", |ui| {
                    if ui.button("Source").clicked() {
                        ctx.open_url(egui::OpenUrl::new_tab("https://git.mcorangehq.xyz/XOR64/music"));
                    }
                    if ui.button("Save").clicked() {
                        if let Err(e) =  self.manifest.save(None) {
                            log::error!("Failed to save manifest: {e}");
                        }
                    }
                    if ui.button("Quit").clicked() {
                        ctx.send_viewport_cmd(egui::ViewportCommand::Close);
                    }
                });

                ui.menu_button("Song", |ui| {
                    if ui.button("Add New").clicked() {
                        self.new_song_w.is_open = true;
                    }
                });

                ui.menu_button("Playlist", |ui| {
                    if ui.button("Import").clicked() {
                        self.import_playlist_w.is_open = true;     
                    }
                });

                ui.menu_button("Downloader", |ui| {
                    if ui.button("Download All").clicked() {
                        if let Err(e) = self.downloader.download_all_nb(&self.manifest, &self.cfg) {
                            log::error!("Err: {e}");
                        }
                    }
                });
                ui.add_space(16.0);
                ui.with_layout(egui::Layout::bottom_up(egui::Align::RIGHT), |ui| {
                    ui.horizontal(|ui| {
                        if self.downloader.get_songs_left_nb() > 0 {
                            self.downloading = true;
                            ui.label(format!("Downloading: {}/{}", self.downloader.get_songs_left_nb(), self.downloader.get_initial_song_count_nb()));
                        } else if self.downloading {
                            let _ = notify_rust::Notification::new()
                                .summary("Done downloading")
                                .body("Your music has been downloaded")
                                .show();
                            self.downloading = false;
                        }
                        let _ = self.downloader.download_all_nb_poll(&self.cfg);
                        egui::widgets::global_dark_light_mode_buttons(ui);
                    });
                });
            });
        });
    }
}
