use std::path::{Path, PathBuf};

use crate::constants::EXEC_EXT;

mod links;

pub struct DepDownloader {
    ytdlp: PathBuf,
    spotdl: PathBuf,
    ffmpeg: PathBuf,
}


impl DepDownloader {
    pub fn new() -> anyhow::Result<Self> {
        if !Path::new(".dep").exists() {
            std::fs::create_dir(".dep")?; 
        }
        
        let ytdlp = Self::download_ytdlp()?;

        todo!()
    }

    fn download_ytdlp() -> anyhow::Result<PathBuf> {
        if let Some(p) = crate::util::is_program_in_path("yt-dlp") {
            return Ok(p); // Already exists on pc
        }
    
        let mut fp = PathBuf::new();
        fp.push(".dep");
        fp.push("yt-dlp");
        fp.set_extension(EXEC_EXT);

        if fp.exists() {
            return Ok(fp);
        }
        
        let mut f = std::fs::OpenOptions::new()
            .create_new(true)
            .write(true)
            .open(&fp)?;

        let mut req = reqwest::blocking::get(links::YTDLP)?;
        req.copy_to(&mut f)?;

        Ok(fp)
    }

    fn download_spotdl() -> anyhow::Result<PathBuf> {
        if let Some(p) = crate::util::is_program_in_path("spotdl") {
            return Ok(p); // already exists on pc
        }
    
        let mut fp = PathBuf::new();
        fp.push(".dep");
        fp.push("spotdl");
        fp.set_extension(EXEC_EXT);

        if fp.exists() {
            return Ok(fp);
        }
        
        if links::SPOTDL.is_empty() {
            log::error!("spotdl doesnt support this architecture (probably aarch64)");
            log::error!("please install it manually, if you have installed it, make sure its accessible in path");
            anyhow::bail!("");
        }

        let mut f = std::fs::OpenOptions::new()
            .create_new(true)
            .write(true)
            .open(&fp)?;

        let mut req = reqwest::blocking::get(links::SPOTDL)?;
        req.copy_to(&mut f)?;

        Ok(fp)
    }

    fn download_ffmpeg() -> anyhow::Result<PathBuf> {
        if let Some(p) = crate::util::is_program_in_path("ffmpeg") {
            return Ok(p); // already exists on pc
        }
    
        let mut fp = PathBuf::new();
        fp.push(".dep");
        fp.push("ffmpeg");
        fp.set_extension(EXEC_EXT);

        if fp.exists() {
            return Ok(fp);
        }
        
        if links::FFMPEG.is_empty() {
            log::error!("spotdl doesnt support this architecture (probably aarch64)");
            log::error!("please install it manually, if you have installed it, make sure its accessible in path");
            anyhow::bail!("");
        }

        let mut f = std::fs::OpenOptions::new()
            .create_new(true)
            .write(true)
            .open(&fp)?;

        let mut req = reqwest::blocking::get(links::FFMPEG)?;
        req.copy_to(&mut f)?;

        Ok(fp)
    }
}
